@@ -26,7 +26,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: threadSpCmd.c,v 1.24 2006/02/09 09:57:59 vasiljevic Exp $
+ * RCS: @(#) $Id: threadSpCmd.c,v 1.32 2010/03/31 08:50:24 vasiljevic Exp $
  * ----------------------------------------------------------------------------
  */
 
@@ -45,6 +45,15 @@
 #define SP_MUTEX   1  /* Any kind of mutex */
 #define SP_CONDV   2  /* The condition variable sync type */
 
+/*
+ * Handle hiding of errorLine in 8.6
+ */
+#if (TCL_MAJOR_VERSION == 8) && (TCL_MINOR_VERSION < 6)
+#define ERRORLINE(interp) ((interp)->errorLine)
+#else
+#define ERRORLINE(interp) (Tcl_GetErrorLine(interp))
+#endif
+
 /* 
  * Structure representing one sync primitive (mutex, condition variable). 
  * We use buckets to manage Tcl names of sync primitives. Each bucket
@@ -120,14 +129,16 @@ static int       SpCondvWait       (SpCondv *, SpMutex *, int);
 static void      SpCondvNotify     (SpCondv *);
 static int       SpCondvFinalize   (SpCondv *);
 
-static void      AddAnyItem        (int, char *, int, SpItem *);
-static SpItem*   GetAnyItem        (int, char *, int);
+static void      AddAnyItem        (int, const char *, int, SpItem *);
+static SpItem*   GetAnyItem        (int, const char *, int);
 static void      PutAnyItem        (SpItem *);
-static int       RemoveMutex       (char *, int);
-static int       RemoveCondv       (char *, int);
+static SpItem *  RemoveAnyItem     (int, const char*, int);
+
+static int       RemoveMutex       (const char *, int);
+static int       RemoveCondv       (const char *, int);
 
 static Tcl_Obj*  GetName           (int, void *);
-static SpBucket* GetBucket         (int, char *, int);
+static SpBucket* GetBucket         (int, const char *, int);
 
 static int       AnyMutexIsLocked  (Sp_AnyMutex *mPtr, Tcl_ThreadId);
 
@@ -179,13 +190,14 @@ ThreadMutexObjCmd(dummy, interp, objc, objv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
     int objc;                           /* Number of arguments. */
-    Tcl_Obj *CONST objv[];              /* Argument objects. */
+    Tcl_Obj *const objv[];              /* Argument objects. */
 {
     int opt, ret, nameLen;
-    char *mutexName, type;
+    const char *mutexName;
+    char type;
     SpMutex *mutexPtr;
 
-    static CONST84 char *cmdOpts[] = {
+    static const char *cmdOpts[] = {
         "create", "destroy", "lock", "unlock", NULL
     };
     enum options {
@@ -216,7 +228,7 @@ ThreadMutexObjCmd(dummy, interp, objc, objv)
 
     if (opt == (int)m_CREATE) {
         Tcl_Obj *nameObj;
-        char *arg;
+        const char *arg;
         
         /*
          * Parse out which type of mutex to create
@@ -351,14 +363,15 @@ ThreadRWMutexObjCmd(dummy, interp, objc, objv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
     int objc;                           /* Number of arguments. */
-    Tcl_Obj *CONST objv[];              /* Argument objects. */
+    Tcl_Obj *const objv[];              /* Argument objects. */
 {
     int opt, ret, nameLen;
-    char *mutexName;
+    const char *mutexName;
     SpMutex *mutexPtr;
     Sp_ReadWriteMutex *rwPtr;
+    Sp_AnyMutex **lockPtr;
 
-    static CONST84 char *cmdOpts[] = {
+    static const char *cmdOpts[] = {
         "create", "destroy", "rlock", "wlock", "unlock", NULL
     };
     enum options {
@@ -453,11 +466,13 @@ ThreadRWMutexObjCmd(dummy, interp, objc, objv)
         return TCL_ERROR;
     }
 
-    rwPtr = (Sp_ReadWriteMutex*)&mutexPtr->lock;
+    lockPtr = &mutexPtr->lock;
+    rwPtr = (Sp_ReadWriteMutex*) lockPtr;
     
     switch ((enum options)opt) {
     case w_RLOCK:
         if (!Sp_ReadWriteMutexRLock(rwPtr)) {
+            PutMutex(mutexPtr);
             Tcl_AppendResult(interp, "read-locking already write-locked mutex ",
                              "from the same thread", NULL);
             return TCL_ERROR;
@@ -510,14 +525,14 @@ ThreadCondObjCmd(dummy, interp, objc, objv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
     int objc;                           /* Number of arguments. */
-    Tcl_Obj *CONST objv[];              /* Argument objects. */
+    Tcl_Obj *const objv[];              /* Argument objects. */
 {
     int opt, ret, nameLen, timeMsec = 0;
-    char *condvName, *mutexName;
+    const char *condvName, *mutexName;
     SpMutex *mutexPtr;
     SpCondv *condvPtr;
 
-    static CONST84 char *cmdOpts[] = {
+    static const char *cmdOpts[] = {
         "create", "destroy", "notify", "wait", NULL
     };
     enum options {
@@ -622,8 +637,8 @@ ThreadCondObjCmd(dummy, interp, objc, objv)
             return TCL_ERROR;
         }
         if (objc == 5) {
-            PutCondv(condvPtr);
             if (Tcl_GetIntFromObj(interp, objv[4], &timeMsec) != TCL_OK) {
+                PutCondv(condvPtr);
                 return TCL_ERROR;
             }
         }
@@ -676,10 +691,10 @@ ThreadEvalObjCmd(dummy, interp, objc, objv)
     ClientData dummy;                   /* Not used. */
     Tcl_Interp *interp;                 /* Current interpreter. */
     int objc;                           /* Number of arguments. */
-    Tcl_Obj *CONST objv[];              /* Argument objects. */
+    Tcl_Obj *const objv[];              /* Argument objects. */
 {
     int ret, optx, internal, nameLen;
-    char *mutexName;
+    const char *mutexName;
     Tcl_Obj *scriptObj;
     SpMutex *mutexPtr = NULL;
     static Sp_RecursiveMutex evalMutex;
@@ -757,8 +772,8 @@ ThreadEvalObjCmd(dummy, interp, objc, objv)
     Tcl_DecrRefCount(scriptObj);
 
     if (ret == TCL_ERROR) {
-        char msg[32 + TCL_INTEGER_SPACE];   
-        sprintf(msg, "\n    (\"eval\" body line %d)", interp->errorLine);
+        char msg[32 + TCL_INTEGER_SPACE];
+        sprintf(msg, "\n    (\"eval\" body line %d)", ERRORLINE(interp));
         Tcl_AddObjErrorInfo(interp, msg, -1);
     }
 
@@ -827,7 +842,7 @@ GetName(int type, void *addrPtr)
  */
 
 static SpBucket*
-GetBucket(int type, char *name, int len)
+GetBucket(int type, const char *name, int len)
 {
     switch (type) {
     case SP_MUTEX: return &muxBuckets[GetHash(name, len)];
@@ -848,13 +863,13 @@ GetBucket(int type, char *name, int len)
  *      Item pointer or NULL
  *
  * Side effects:
- *      None.
+ *      Increment the item's ref count preventing it's deletion.
  *
  *----------------------------------------------------------------------
  */
 
 static SpItem*
-GetAnyItem(int type, char *name, int len)
+GetAnyItem(int type, const char *name, int len)
 {
     SpItem *itemPtr = NULL;
     SpBucket *bucketPtr = GetBucket(type, name, len);
@@ -864,6 +879,7 @@ GetAnyItem(int type, char *name, int len)
     hashEntryPtr = Tcl_FindHashEntry(&bucketPtr->handles, name);
     if (hashEntryPtr != (Tcl_HashEntry*)NULL) {
         itemPtr = (SpItem*)Tcl_GetHashValue(hashEntryPtr);
+        itemPtr->refcnt++;
     }
     Tcl_MutexUnlock(&bucketPtr->lock);
 
@@ -875,13 +891,14 @@ GetAnyItem(int type, char *name, int len)
  *
  * PutAnyItem --
  *
- *      Returns the item back.
+ *      Current thread detaches from the item.
  *
  * Results:
  *      None.
  *
  * Side effects:
- *      None.
+ *      Decrement item's ref count allowing for it's deletion
+ *      and signalize any threads waiting to delete the item.
  *
  *----------------------------------------------------------------------
  */
@@ -891,6 +908,7 @@ PutAnyItem(SpItem *itemPtr)
 {
     Tcl_MutexLock(&itemPtr->bucket->lock);
     itemPtr->refcnt--;
+    Tcl_ConditionNotify(&itemPtr->bucket->cond);
     Tcl_MutexUnlock(&itemPtr->bucket->lock);
 }
 
@@ -911,7 +929,7 @@ PutAnyItem(SpItem *itemPtr)
  */
 
 static void
-AddAnyItem(int type, char *handle, int len, SpItem *itemPtr)
+AddAnyItem(int type, const char *handle, int len, SpItem *itemPtr)
 {
     int new;
     SpBucket *bucketPtr = GetBucket(type, handle, len);
@@ -929,16 +947,55 @@ AddAnyItem(int type, char *handle, int len, SpItem *itemPtr)
     Tcl_MutexUnlock(&bucketPtr->lock);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * RemoveAnyItem --
+ *
+ *      Removes the item from it's bucket.
+ *
+ * Results:
+ *      Item's pointer or NULL if none found.
+ *
+ * Side effects:
+ *      None.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static SpItem *
+RemoveAnyItem(int type, const char *name, int len)
+{
+    SpItem *itemPtr = NULL;
+    SpBucket *bucketPtr = GetBucket(type, name, len);
+    Tcl_HashEntry *hashEntryPtr = NULL;
+
+    Tcl_MutexLock(&bucketPtr->lock);
+    hashEntryPtr = Tcl_FindHashEntry(&bucketPtr->handles, name);
+    if (hashEntryPtr == (Tcl_HashEntry*)NULL) {
+        Tcl_MutexUnlock(&bucketPtr->lock);
+        return NULL;
+    }
+    itemPtr = (SpItem*)Tcl_GetHashValue(hashEntryPtr);
+    Tcl_DeleteHashEntry(hashEntryPtr);
+    while (itemPtr->refcnt > 0) {
+        Tcl_ConditionWait(&bucketPtr->cond, &bucketPtr->lock, NULL);
+    }
+    Tcl_MutexUnlock(&bucketPtr->lock);
+
+    return itemPtr;
+}
+
 /*
  *----------------------------------------------------------------------
  *
  * RemoveMutex --
  *
- *      Finalizes the mutex and removes it from it's bucket.
+ *      Removes the mutex from it's bucket and finalizes it.
  *
  * Results:
  *      1 - mutex is finalized and removed
- *      0 - mutex is not removed (still in use)
+ *      0 - mutex is not finalized
  +     -1 - mutex is not found
  *
  * Side effects:
@@ -948,26 +1005,19 @@ AddAnyItem(int type, char *handle, int len, SpItem *itemPtr)
  */
 
 static int
-RemoveMutex(char *name, int len)
+RemoveMutex(const char *name, int len)
 {
-    SpMutex *mutexPtr = NULL;
-    SpBucket *bucketPtr = GetBucket(SP_MUTEX, name, len);
-    Tcl_HashEntry *hashEntryPtr = NULL;
-
-    Tcl_MutexLock(&bucketPtr->lock);
-    hashEntryPtr = Tcl_FindHashEntry(&bucketPtr->handles, name);
-    if (hashEntryPtr == (Tcl_HashEntry*)NULL) {
-        Tcl_MutexUnlock(&bucketPtr->lock);
-        return -1; /* Mutex not found */
+    SpMutex *mutexPtr = GetMutex(name, len);
+    if (mutexPtr == NULL) {
+        return -1;
     }
-    mutexPtr = (SpMutex*)Tcl_GetHashValue(hashEntryPtr);
-    if (mutexPtr->refcnt > 0 || SpMutexFinalize(mutexPtr) == 0) {
-        Tcl_MutexUnlock(&bucketPtr->lock);
-        return 0; /* Could not finalize, in use? */
+    if (!SpMutexFinalize(mutexPtr)) {
+        PutMutex(mutexPtr);
+        return 0;
     }
+    PutMutex(mutexPtr);
+    RemoveAnyItem(SP_MUTEX, name, len);
     Tcl_Free((char*)mutexPtr);
-    Tcl_DeleteHashEntry(hashEntryPtr);
-    Tcl_MutexUnlock(&bucketPtr->lock);
 
     return 1;
 }
@@ -977,11 +1027,11 @@ RemoveMutex(char *name, int len)
  *
  * RemoveCondv --
  *
- *      Finalizes the condition variable and removes it from it's bucket.
+ *      Removes the cond variable from it's bucket and finalizes it.
  *
  * Results:
  *      1 - variable is finalized and removed
- *      0 - variable is not removed (still in use)
+ *      0 - variable is not finalized
  +     -1 - variable is not found
  *
  * Side effects:
@@ -991,26 +1041,19 @@ RemoveMutex(char *name, int len)
  */
 
 static int
-RemoveCondv(char *name, int len)
+RemoveCondv(const char *name, int len)
 {
-    SpCondv *condvPtr = NULL;
-    SpBucket *bucketPtr = GetBucket(SP_CONDV, name, len);
-    Tcl_HashEntry *hashEntryPtr = NULL;
-
-    Tcl_MutexLock(&bucketPtr->lock);
-    hashEntryPtr = Tcl_FindHashEntry(&bucketPtr->handles, name);
-    if (hashEntryPtr == (Tcl_HashEntry*)NULL) {
-        Tcl_MutexUnlock(&bucketPtr->lock);
-        return -1; /* Variable not found */
+    SpCondv *condvPtr = GetCondv(name, len);
+    if (condvPtr == NULL) {
+        return -1;
     }
-    condvPtr = (SpCondv*)Tcl_GetHashValue(hashEntryPtr);
-    if (condvPtr->refcnt > 0 || SpCondvFinalize(condvPtr) == 0) {
-        Tcl_MutexUnlock(&bucketPtr->lock);
-        return 0; /* Could not finalize, in use? */
+    if (!SpCondvFinalize(condvPtr)) {
+        PutCondv(condvPtr);
+        return 0;
     }
+    PutCondv(condvPtr);
+    RemoveAnyItem(SP_CONDV, name, len);
     Tcl_Free((char*)condvPtr);
-    Tcl_DeleteHashEntry(hashEntryPtr);
-    Tcl_MutexUnlock(&bucketPtr->lock);
 
     return 1;
 }
@@ -1055,10 +1098,10 @@ Sp_Init (interp)
         Tcl_MutexUnlock(&initMutex);
     }
 
-    TCL_CMD(interp, THNS"::mutex",   ThreadMutexObjCmd);
-    TCL_CMD(interp, THNS"::rwmutex", ThreadRWMutexObjCmd);
-    TCL_CMD(interp, THNS"::cond",    ThreadCondObjCmd);
-    TCL_CMD(interp, THNS"::eval",    ThreadEvalObjCmd);
+    TCL_CMD(interp, THREAD_CMD_PREFIX"::mutex",   ThreadMutexObjCmd);
+    TCL_CMD(interp, THREAD_CMD_PREFIX"::rwmutex", ThreadRWMutexObjCmd);
+    TCL_CMD(interp, THREAD_CMD_PREFIX"::cond",    ThreadCondObjCmd);
+    TCL_CMD(interp, THREAD_CMD_PREFIX"::eval",    ThreadEvalObjCmd);
 
     return TCL_OK;
 }
@@ -1083,12 +1126,14 @@ Sp_Init (interp)
 static int 
 SpMutexLock(SpMutex *mutexPtr)
 {
+    Sp_AnyMutex **lockPtr = &mutexPtr->lock;
+
     switch (mutexPtr->type) {
     case EMUTEXID:
-        return Sp_ExclusiveMutexLock((Sp_ExclusiveMutex*)&mutexPtr->lock);
+        return Sp_ExclusiveMutexLock((Sp_ExclusiveMutex*)lockPtr);
         break;
     case RMUTEXID: 
-        return Sp_RecursiveMutexLock((Sp_RecursiveMutex*)&mutexPtr->lock);
+        return Sp_RecursiveMutexLock((Sp_RecursiveMutex*)lockPtr);
         break;
     }
 
@@ -1115,12 +1160,14 @@ SpMutexLock(SpMutex *mutexPtr)
 static int
 SpMutexUnlock(SpMutex *mutexPtr)
 {
+    Sp_AnyMutex **lockPtr = &mutexPtr->lock;
+
     switch (mutexPtr->type) {
     case EMUTEXID:
-        return Sp_ExclusiveMutexUnlock((Sp_ExclusiveMutex*)&mutexPtr->lock);
+        return Sp_ExclusiveMutexUnlock((Sp_ExclusiveMutex*)lockPtr);
         break;
     case RMUTEXID:
-        return Sp_RecursiveMutexUnlock((Sp_RecursiveMutex*)&mutexPtr->lock);
+        return Sp_RecursiveMutexUnlock((Sp_RecursiveMutex*)lockPtr);
         break;
     }
 
@@ -1148,6 +1195,8 @@ SpMutexUnlock(SpMutex *mutexPtr)
 static int
 SpMutexFinalize(SpMutex *mutexPtr)
 {
+    Sp_AnyMutex **lockPtr = &mutexPtr->lock;
+
     if (AnyMutexIsLocked((Sp_AnyMutex*)mutexPtr->lock, (Tcl_ThreadId)0)) {
         return 0;
     }
@@ -1159,13 +1208,13 @@ SpMutexFinalize(SpMutex *mutexPtr)
 
     switch (mutexPtr->type) {
     case EMUTEXID:
-        Sp_ExclusiveMutexFinalize((Sp_ExclusiveMutex*)&mutexPtr->lock);
+        Sp_ExclusiveMutexFinalize((Sp_ExclusiveMutex*)lockPtr);
         break;
     case RMUTEXID:
-        Sp_RecursiveMutexFinalize((Sp_RecursiveMutex*)&mutexPtr->lock);
+        Sp_RecursiveMutexFinalize((Sp_RecursiveMutex*)lockPtr);
         break;
     case WMUTEXID:
-        Sp_ReadWriteMutexFinalize((Sp_ReadWriteMutex*)&mutexPtr->lock);
+        Sp_ReadWriteMutexFinalize((Sp_ReadWriteMutex*)lockPtr);
         break;
     default:
         break;
@@ -1194,7 +1243,8 @@ SpMutexFinalize(SpMutex *mutexPtr)
 static int
 SpCondvWait(SpCondv *condvPtr, SpMutex *mutexPtr, int msec)
 {
-    Sp_ExclusiveMutex_ *emPtr = *(Sp_ExclusiveMutex_**)&mutexPtr->lock;
+	Sp_AnyMutex **lock = &mutexPtr->lock;
+    Sp_ExclusiveMutex_ *emPtr = *(Sp_ExclusiveMutex_**)lock;
     Tcl_Time waitTime, *wt = NULL;
     Tcl_ThreadId threadId = Tcl_GetCurrentThread();
 
@@ -1664,7 +1714,7 @@ Sp_ReadWriteMutexRLock(Sp_ReadWriteMutex *muxPtr)
         Tcl_MutexUnlock(&rwPtr->lock);
         return 0; /* We already hold the write lock */
     }
-    while (rwPtr->lockcount < 0 || rwPtr->numwr > 0) {
+    while (rwPtr->lockcount < 0) {
         rwPtr->numrd++;
         Tcl_ConditionWait(&rwPtr->rcond, &rwPtr->lock, NULL);
         rwPtr->numrd--;
